// *****************************************************************************
// Module de gestion de l'automate
// *****************************************************************************
#include <reg51f.h>
#include <stdio.h>
#include "chrono.h"
#include "stepper.h"
#include "automat.h"
#include "stepper.h"
#include "cmde.h"

// *****************************************************************************
// variables globales du module
// *****************************************************************************
unsigned int code ptr_start_pgm[16] _at_ 0x7F00;
unsigned char code pgm_data[0x8000] _at_ 0x8000;

unsigned int pgm_pc=0;
bit	running=0;

unsigned char xdata stack[32];
unsigned char stack_ptr=0;

bit debug=0;
extern bit err;


// *****************************************************************************
void gere_automate(void)
{
unsigned int x;
char d,v;

if (!running) return;
if (rebours_to_do) return;							// il faut revenir plus tard (le compte  rebours n'est pas termin)

switch(pgm_data[pgm_pc++])
	{

	case	CMDE_MOT:	d=pgm_data[pgm_pc++];
						if(debug)
							{
							printf("\nMise en route moteur: ");
							if (d&0x01)
								{
								printf(" DC1:");
								if (d&0x02) printf("+");
								else printf("-");
								}

							if (d&0x04)
								{
								printf(" DC2:");
								if (d&0x08) printf("+");
								else printf("-");
								}
							}

						if (d&0x01)
							{
							if (d&0x02)
								{
								// met le moteur dans le sens +
								m1a=1;
								m1b=0;
								}
							else
								{
								// met le moteur dans le sens -
								m1a=0;
								m1b=1;
								}
							en1=1;	// met le moteur en route
							}

						if (d&0x04)
							{
							if (d&0x08)
								{
								// met le moteur dans le sens +
								m2a=1;
								m2b=0;
								}
							else
								{
								// met le moteur dans le sens -
								m2a=0;
								m2b=1;
								}
							en1=en2=1;	// met le moteur en route
							}
						break;

	case	CMDE_STOP:	d=pgm_data[pgm_pc++];
						if(debug)
							{
							printf("\nArret moteur:");
							if (d&0x01) printf(" DC1");
							if (d&0x02) printf(" DC2");
							if (d&0x04) printf(" pas  pas");
							}
						if (d&0x01)
							{
							m1a=0;
							m1b=0;
							}
						if (d&0x02)
							{
							m2a=0;
							m2b=0;
							}
						if (d&0x04)
							{
							stop_motor_a();
							}
						break;

	case	CMDE_IDLE:	d=pgm_data[pgm_pc++];
						if(debug)
							{
							printf("\nRoue libre moteur:");
							if (d&0x01) printf(" DC1");
							if (d&0x02) printf(" DC2");
							if (d&0x04) printf(" pas  pas");
							}
						if (d&0x01) en1=0;
						if (d&0x02) en2=0;
						if (d&0x04) en3=en4=0;
						break;


	case	CMDE_STEPPER: 
						d=pgm_data[pgm_pc++];		// charge le nb de pas
						v=pgm_data[pgm_pc++];		// charge la temporisation entre les pas (vitesse)
						if(debug)
							{
							printf("\nMoteur pas  pas: %bd (%dms)",d,v);
							}
						speed_motor_a(v);
						step_motor_a(d);
						break;


	case	WAIT:		x=256*pgm_data[pgm_pc++]+pgm_data[pgm_pc++];
						if(debug)
							{
							printf("\nAttente: %u ms",x);
							}
						start_rebours(x);								// lance le compte  rebours pour la dure demande
						break;

	case	CALL:		x=(pgm_data[pgm_pc++]<<8)+pgm_data[pgm_pc++];	// determine l'adresse du saut
						if(debug)
							{
							printf("\nAppel du sous programme : %u (niveau:%bd)",x,stack_ptr);
							}
						if (stack_ptr>=31)			// vrifie si il reste de la place sur la pile
							{
							printf("\nDpassement d'appels (32 niveaux max)");
							running=0;
							err=1;
							return;
							}
						stack[stack_ptr++]=pgm_pc;	// mmorise l'adresse de retour
						pgm_pc=x;					// saut  la nouvelle adresse
						break;

	case	RETURN:		if(debug) printf("\nRetour de sous programme");
						if (stack_ptr==0)
							{
							printf("\nRetour impossible, pile vide");
							running=0;
							err=1;
							return;
							}
						pgm_pc=stack[stack_ptr-1];	// charge l'adresse de retour
						stack_ptr--;
						break;

	case	JUMP:		x=((pgm_data[pgm_pc++])<<8)+pgm_data[pgm_pc++];	// determine l'adresse du saut
						if(debug)
							{
							printf("\nSaut direct : %u",x);
							}
						pgm_pc=x;
						break;

	case	DBG:		if (pgm_data[pgm_pc++])
							{
							debug=1;
							printf("\nMode debug = ON");
							}
						else
							{
							debug=0;
							printf("\nMode debug = OFF");
							}
						break;

	case	CMDE_END:	if(debug) printf("\nFin du programme");
						running=0;
						break;

	default:			printf("\nFin anormale du programme (OP code inconnu = %b02X)",pgm_data[pgm_pc-1]);
						running=0;
						err=1;
						break;

	}

if (!running)
	{
	en1=en2=en3=en4=0;			// repasse tous les moteurs en roue libre
	}

// vrifie si le pointeur de programme est toujours valide
// (mini = 3 octets -> 2 octets pour le n de ligne, 1 octet pour l'opcode)
if (x>0x7FFC)
	{
	running=0;
	err=1;
	printf("\nDpassement PC");
	}
}
