// ****************************************************************************
// (C) Morin Pascal 2002
// ****************************************************************************

#include <reg51f.h>
#include <stdio.h>
#include "chrono.h"
#include "stepper.h"
#include "bpgetkey.h"
#include "automat.h"
#include "boot.h"


#define F12MHZ

// ****************************************************************************
// definition des ports d'entree et de sortie du microcontroleur
// ****************************************************************************
sbit LED1=P1^1;
sbit LED2=P1^0;


// ****************************************************************************
// equivalences generales
// ****************************************************************************
#define byte		unsigned char
#define masq_sw		0x0F

// ****************************************************************************
// variables globales
// ****************************************************************************
bit 			wait_to_do=0;
unsigned int	wait_count;
bit err=0;



// ***************************************************************************
void wait(int x)
{
wait_count=x;
wait_to_do=1;
while(wait_to_do);
}

// ***************************************************************************
// gestion du temps (int toutes les 1ms environ)
// ***************************************************************************
gestion_timer0() interrupt 1 using 2
{
// --------------------------------------------------
// recharge la valeur du timer (rendez-vous dans 1ms)
// --------------------------------------------------
#ifdef F12MHZ
TH0=0xfc;				// FC18 -> 1ms  12MHz
TL0=0x18;
#else
TH0=0xFC;				// FC66 -> 1ms  a 11.059MHz
TL0=0x66;
#endif

if (wait_to_do)
	{
   if(--wait_count ==0) wait_to_do=0;
   }
gere_chrono();
gere_stepper();
}


// ***************************************************************************
// point d'entree du programme
// ***************************************************************************
void main(void)
{
unsigned char x;

// -------------------------
// initialisation des timers
// -------------------------
TMOD=0x21;								// timer0 mode compteur 16 bits
										// timer1 8 bits auto-reload (baud-rate pour 11,925MHz)
#ifdef F12MHZ
TH0=0xfc;								// FC18 -> 1ms
TL0=0x18;
#else
TH0=0xFC;								// FC66 -> 1ms  a 11.059Mhz
TL0=0x66;
#endif

// ----------------------------
// initialisation du port serie
// ----------------------------
SCON=0x50;								//serial mode 1 (8bits none 1stop), receive enable
#ifdef F12MHZ
RCAP2H=0xff;							// FFD8 pour 9600 Bauds (horloge  12MHZ)
RCAP2L=0xd8;
T2CON=0x34;								// baud rate generator (RCLK+TCLK+TR2)
#else
TH1=0xFD;								// 9600 bauds avec un quartz de 11,9250MHz
TR1=1;
#endif
TI=1;									// pour autoriser prochain envoi


TR0=1; ET0=1; 							// timer 0 mis en route, int autorise
EA=1;									// interruptions autorisees


// ----------------------------
// affiche le message d'acceuil
// ----------------------------
printf("\nAutomate pour commande de 2 moteurs DC et un moteur pas  pas\nVersion 1.0\n(C)Morin Pascal 2002");

init_stepper();


// ----------------------------
// boucle principale
// ----------------------------
while(1)
	{
	if (err)
		{
		LED1=0;
		err=0;
		}

	// vrifie si il y a une action sur BP1  prendre en compte
	if (bp_kbhit())		
		{
		if (running)
			{
			running=0;
			LED1=1;
			LED2=1;

			// arrete les moteurs
			m1a=0;
			m2a=0;
			m1b=0;
			m2b=0;
			stop_motor_a();
			wait(100);

			// passe les moteurs en roue libre
			en1=en2=en3=en4=0;

			}
		else
			{
			running=1;
			LED1=1;
			LED2=0;
			x=(P3>>3)&masq_sw;			// lecture du code impos par sw1
			pgm_pc=ptr_start_pgm[x];	// determine l'adresse de dpart du programme
			}
		bp_get_key();	// attend que la touche soit relache
		}


	// vrifie si une commande arrive par la liaison RS232
	if (RI)
		{
		switch(getchar())
			{
			case'#': 	boot();
						break;

			case'*':	debug=1;
						printf("\nMode debug activ");
						break;

			case'$':	debug=0;
						printf("\nMode debug dsactiv");
						break;
			}
		}
	gere_automate();
	}
}

